

#include "context.hpp"

// awfull but necessary
extern "C"
{
#include <krb5.h>
#include <admin.h>
}


py_kadm5::kcontext::kcontext(const boost::python::object& error_callback)
:  context_(0), err_cb_(error_callback)
{
    if (krb5_error_code e =
            krb5_init_context(&context_))
        throw_exception(e, "while initializing kerberos library");
}

py_kadm5::kcontext::kcontext(krb5_context other,
        const boost::python::object& error_callback)
: context_(other), err_cb_(error_callback)
{
    // nothing to do
}


py_kadm5::kcontext::~kcontext()
{
    if (context_)
        krb5_free_context(context_);
}

void
py_kadm5::kcontext::reset(krb5_context other)
{
    if (context_)
        krb5_free_context(context_);
    context_ = other;
}

namespace
{
    static inline
    void raise(PyObject* exc_type, const boost::python::object& value)
    {
        PyErr_SetObject(exc_type, value.ptr());
        boost::python::throw_error_already_set();
    }

    static inline
    void raise(const boost::python::object& obj)
    {
        raise(PyObject_Type(obj.ptr()), obj);
    }

}

void
py_kadm5::kcontext::throw_exception(long code, const char *where)
{
    using namespace boost::python;

    if (PyCallable_Check(err_cb_.ptr()))
    {
        raise(err_cb_(code, where, error_message(code)));
    }

    object exc_value(make_tuple(code, where, error_message(code)));

    if (err_cb_.ptr() != Py_None)
    {
        exc_value += make_tuple("WARNING! Bad error callback supplied");
    }

    raise(PyExc_RuntimeError, exc_value);
}

