
#ifndef PY_KRB5_CONTEXT_HPP
#define PY_KRB5_CONTEXT_HPP

#include <boost/python.hpp>
#include <boost/noncopyable.hpp>

//forward-declare some data type from krb5.h
struct _krb5_context;
typedef struct _krb5_context *krb5_context;


namespace py_kadm5
{
    class kcontext : boost::noncopyable
    {
        public:
            /// initialize context with krb5_init_context()
            kcontext(const boost::python::object& error_callback);


            /// take ownership of context
            kcontext(krb5_context other,
                 const boost::python::object& error_callback);

            ~kcontext();

            /// frees current context and takes ownership of other
            void reset(krb5_context other = 0);

            /// accessor
            krb5_context context() const
            {
                return context_;
            }

            /**
             * Throw Python exception from kerberos error code.
             *
             * Sets python exception of type @c err_cb_ and provides it 
             * a 3-tuple as an argument: error code, user-supplied string
             * @c where and error message from kerberos library.
             *
             * @param code: error code.
             * @param where: additional message passed to exception.
             */
            void throw_exception(long code, const char *where);



        private:
            krb5_context context_;
            boost::python::object err_cb_;
    };
}

#endif //PY_KRB5_CONTEXT_HPP

