
#ifndef PY_KADMIN5_HPP
#define PY_KADMIN5_HPP

#include <string>
#include <boost/python.hpp>
#include <boost/noncopyable.hpp>
#include "context.hpp"

//forward-declare some data type from krb5.h
struct _krb5_context;
typedef struct _krb5_context *krb5_context;

struct krb5_principal_data;
typedef struct krb5_principal_data *krb5_principal;


namespace py_kadm5
{
    /**
     * Kerberos 5 database administration utility for python.
     *
     * The class is used to provide Kerberos 5 database administration
     * API bindings for python via boost::python.
     *
     * For additional documentation see docstrings defined in modile.cpp
     */
    class kadmin: boost::noncopyable
    {
        public:
            kadmin(boost::python::object error_callback
                    , const char* name
                    , const char* realm
                  );

            ~kadmin();

            ///  Retrieve all or some principal names
            boost::python::list list(const char* expr);

            std::string create_princ(const char* name, const char* pass,
                    boost::python::dict args);

            std::string create_service_princ(const char* svc, const char* host,
                    boost::python::dict args);

            void delete_princ(const char* princ_name);

            void chpass_princ(const char* princ_name, const char* pass);

            boost::python::list get_princ_keys(const char* princ_name);

            void randkey_princ(const char* princ_name);

            int get_princ_attributes(const char* princ_name);

            void set_princ_attributes(const char* princ_name, int attrs);

            std::string get_realm();

        private:
            void throw_exception(long code, const char *where)
            {
                return cntx_.throw_exception(code, where);
            }

            /// frees the resources
            void cleanup();

            /// a wrapper around krb5_parse_name which calls append_realm
            int parse_principal(std::string& name, krb5_principal* pprinc);

            krb5_context context()
            {
                return cntx_.context();
            }

            void* handle_;
            kcontext cntx_;
            std::string realm_;
    };
};

#endif //PY_KADMIN5_HPP

