
#include <boost/python.hpp>
#include "kadmin5.hpp"
#include "kadm_err.h"
#include "keytab.hpp"
#include <iostream>

extern "C"
{
#include <krb5.h>
#include <kdb.h>
};
using namespace boost::python;

namespace py_kadm5
{
class kadm_errors {};
class krb5_flags {};
}

BOOST_PYTHON_MODULE(kadmin5)
{

    class_<py_kadm5::krb5_flags, boost::noncopyable>(
              "attributes"
            , "krb5_flags that denote possible principal attributes\n"
            ,  no_init)
        .setattr("DISALLOW_POSTDATED", KRB5_KDB_DISALLOW_POSTDATED)
        .setattr("DISALLOW_FORWARDABLE", KRB5_KDB_DISALLOW_FORWARDABLE)
        .setattr("DISALLOW_TGT_BASED", KRB5_KDB_DISALLOW_TGT_BASED)
        .setattr("DISALLOW_RENEWABLE", KRB5_KDB_DISALLOW_RENEWABLE)
        .setattr("DISALLOW_PROXIABLE", KRB5_KDB_DISALLOW_PROXIABLE)
        .setattr("DISALLOW_DUP_SKEY", KRB5_KDB_DISALLOW_DUP_SKEY)
        .setattr("DISALLOW_ALL_TIX", KRB5_KDB_DISALLOW_ALL_TIX)
        .setattr("REQUIRES_PRE_AUTH", KRB5_KDB_REQUIRES_PRE_AUTH)
        .setattr("REQUIRES_HW_AUTH", KRB5_KDB_REQUIRES_HW_AUTH)
        .setattr("REQUIRES_PWCHANGE", KRB5_KDB_REQUIRES_PWCHANGE)
        .setattr("DISALLOW_SVR", KRB5_KDB_DISALLOW_SVR)
        .setattr("PWCHANGE_SERVICE", KRB5_KDB_PWCHANGE_SERVICE)
        .setattr("SUPPORT_DESMD5", KRB5_KDB_SUPPORT_DESMD5)
        .setattr("NEW_PRINC", KRB5_KDB_NEW_PRINC)
        ;

    class_<py_kadm5::krb5_flags, boost::noncopyable>(
              "kadm_errors"
            ,  no_init)
        .setattr("KADM5_FAILURE", KADM5_FAILURE) // Operation failed for unspecified reason
        .setattr("KADM5_AUTH_GET", KADM5_AUTH_GET) // Operation requires ``get'' privilege
        .setattr("KADM5_AUTH_ADD", KADM5_AUTH_ADD) // Operation requires ``add'' privilege
        .setattr("KADM5_AUTH_MODIFY", KADM5_AUTH_MODIFY) // Operation requires ``modify'' privilege
        .setattr("KADM5_AUTH_DELETE", KADM5_AUTH_DELETE) // Operation requires ``delete'' privilege
        .setattr("KADM5_AUTH_INSUFFICIENT", KADM5_AUTH_INSUFFICIENT) // Insufficient authorization for operation
        .setattr("KADM5_BAD_DB", KADM5_BAD_DB) // Database inconsistency detected
        .setattr("KADM5_DUP", KADM5_DUP) // Principal or policy already exists
        .setattr("KADM5_RPC_ERROR", KADM5_RPC_ERROR) // Communication failure with server
        .setattr("KADM5_NO_SRV", KADM5_NO_SRV) // No administration server found for realm
        .setattr("KADM5_BAD_HIST_KEY", KADM5_BAD_HIST_KEY) // Password history principal key version mismatch
        .setattr("KADM5_NOT_INIT", KADM5_NOT_INIT) // Connection to server not initialized
        .setattr("KADM5_UNK_PRINC", KADM5_UNK_PRINC) // Principal does not exist
        .setattr("KADM5_UNK_POLICY", KADM5_UNK_POLICY) // Policy does not exist
        .setattr("KADM5_BAD_MASK", KADM5_BAD_MASK) // Invalid field mask for operation
        .setattr("KADM5_BAD_CLASS", KADM5_BAD_CLASS) // Invalid number of character classes
        .setattr("KADM5_BAD_LENGTH", KADM5_BAD_LENGTH) // Invalid password length
        .setattr("KADM5_BAD_POLICY", KADM5_BAD_POLICY) // Illegal policy name
        .setattr("KADM5_BAD_PRINCIPAL", KADM5_BAD_PRINCIPAL) // Illegal principal name
        .setattr("KADM5_BAD_AUX_ATTR", KADM5_BAD_AUX_ATTR) // Invalid auxillary attributes
        .setattr("KADM5_BAD_HISTORY", KADM5_BAD_HISTORY) // Invalid password history count
        .setattr("KADM5_BAD_MIN_PASS_LIFE", KADM5_BAD_MIN_PASS_LIFE) // Password minimum life is greater than password maximum life
        .setattr("KADM5_PASS_Q_TOOSHORT", KADM5_PASS_Q_TOOSHORT) // Password is too short
        .setattr("KADM5_PASS_Q_CLASS", KADM5_PASS_Q_CLASS) // Password does not contain enough character classes
        .setattr("KADM5_PASS_Q_DICT", KADM5_PASS_Q_DICT) // Password is in the password dictionary
        .setattr("KADM5_PASS_REUSE", KADM5_PASS_REUSE) // Cannot reuse password
        .setattr("KADM5_PASS_TOOSOON", KADM5_PASS_TOOSOON) // Current password's minimum life has not expired
        .setattr("KADM5_POLICY_REF", KADM5_POLICY_REF) // Policy is in use
        .setattr("KADM5_INIT", KADM5_INIT) // Connection to server already initialized
        .setattr("KADM5_BAD_PASSWORD", KADM5_BAD_PASSWORD) // Incorrect password
        .setattr("KADM5_PROTECT_PRINCIPAL", KADM5_PROTECT_PRINCIPAL) // Cannot change protected principal
        .setattr("KADM5_BAD_SERVER_HANDLE", KADM5_BAD_SERVER_HANDLE) // Programmer error!  Bad Admin server handle
        .setattr("KADM5_BAD_STRUCT_VERSION", KADM5_BAD_STRUCT_VERSION) // Programmer error!  Bad API structure version
        .setattr("KADM5_OLD_STRUCT_VERSION", KADM5_OLD_STRUCT_VERSION) // API structure version specified by application is no longer supported (to fix, recompile application against current KADM5 API header file
        .setattr("KADM5_NEW_STRUCT_VERSION", KADM5_NEW_STRUCT_VERSION) // API structure version specified by application is unknown to libraries (to fix, obtain current KADM5 API header files and libraries and rec
        .setattr("KADM5_BAD_API_VERSION", KADM5_BAD_API_VERSION) // Programmer error!  Bad API version
        .setattr("KADM5_OLD_LIB_API_VERSION", KADM5_OLD_LIB_API_VERSION) // API version specified by application is no longer supported by libraries (to fix, update application to adhere to current API version and 
        .setattr("KADM5_OLD_SERVER_API_VERSION", KADM5_OLD_SERVER_API_VERSION) // API version specified by application is no longer supported by server (to fix, update application to adhere to current API version and 
        .setattr("KADM5_NEW_LIB_API_VERSION", KADM5_NEW_LIB_API_VERSION) // API version specified by application is unknown to libraries (to fix, obtain current KADM5 API header files and libraries and recompile ap
        .setattr("KADM5_NEW_SERVER_API_VERSION", KADM5_NEW_SERVER_API_VERSION) // API version specified by application is unknown to server (to fix, obtain and install newest KADM5 Admin Server)
        .setattr("KADM5_SECURE_PRINC_MISSING", KADM5_SECURE_PRINC_MISSING) // Database error! Required KADM5 principal missing
        .setattr("KADM5_NO_RENAME_SALT", KADM5_NO_RENAME_SALT) // The salt type of the specified principal does not support renaming
        .setattr("KADM5_BAD_CLIENT_PARAMS", KADM5_BAD_CLIENT_PARAMS) // Illegal configuration parameter for remote KADM5 client
        .setattr("KADM5_BAD_SERVER_PARAMS", KADM5_BAD_SERVER_PARAMS) // Illegal configuration parameter for local KADM5 client
        .setattr("KADM5_AUTH_LIST", KADM5_AUTH_LIST) // Operation requires ``list'' privilege
        .setattr("KADM5_AUTH_CHANGEPW", KADM5_AUTH_CHANGEPW) // Operation requires ``change-password'' privilege
        .setattr("KADM5_GSS_ERROR", KADM5_GSS_ERROR) // GSS-API (or Kerberos) error
        .setattr("KADM5_BAD_TL_TYPE", KADM5_BAD_TL_TYPE) // Programmer error!  Illegal tagged data list type
        .setattr("KADM5_MISSING_CONF_PARAMS", KADM5_MISSING_CONF_PARAMS) // Required parameters in kdc.conf missing
        .setattr("KADM5_BAD_SERVER_NAME", KADM5_BAD_SERVER_NAME) // Bad krb5 admin server hostname
        .setattr("KADM5_AUTH_SETKEY", KADM5_AUTH_SETKEY) // Operation requires ``set-key'' privilege
        .setattr("KADM5_SETKEY_DUP_ENCTYPES", KADM5_SETKEY_DUP_ENCTYPES) // Multiple values for single or folded enctype
        .setattr("KADM5_SETV4KEY_INVAL_ENCTYPE", KADM5_SETV4KEY_INVAL_ENCTYPE) // Invalid enctype for setv4key
        .setattr("KADM5_SETKEY3_ETYPE_MISMATCH", KADM5_SETKEY3_ETYPE_MISMATCH) // Mismatched enctypes for setkey3
        .setattr("KADM5_MISSING_KRB5_CONF_PARAMS", KADM5_MISSING_KRB5_CONF_PARAMS) // Missing parameters in krb5.conf required for kadmin client
        .setattr("KADM5_XDR_FAILURE", KADM5_XDR_FAILURE) // XDR encoding error
        ;

    class_<py_kadm5::kadmin, boost::noncopyable>(
              "kadmin"
            , "Kerberos 5 database administration utility."
            , init<object, const char*, const char*>(
                  "initializes kadmin object"
                , (
                      arg("error_cb") = object() // None
                    , arg("princname")= "kamin/admin"
                    , arg("realm") = (const char*) 0
                  )
                )
            )
        .def("list"
                , &py_kadm5::kadmin::list
                , "Retrieve all or some principal names.\n\n"
                  "The only argument of this method is an expression. "
                  "Expression is a shell-style glob expression that can "
                  "contain the wild-card characters ?, *, and []'s. "
                  "All principal names matching the  expression are returned "
                  "in a list. If no expression is provided, list of all "
                  "principal names is returned.\n"
                , (arg("expr") = (const char*)0)
            )
        .def("delete_princ"
                , &py_kadm5::kadmin::delete_princ
                , "Remove principal from database.\n\n"
                  "The only argument of this method is principal name.\n"
            )
        .def("randkey_princ"
                , &py_kadm5::kadmin::randkey_princ
                , "Generate new random key for principal.\n\n"
                  "For given principal, new random key is generated.\n"
            )
        .def("chpass_princ"
                , &py_kadm5::kadmin::chpass_princ
                , "Change a password for a principal.\n\n"
            )
        .def("get_princ_attributes"
                , &py_kadm5::kadmin::get_princ_attributes
                , "Get attributes of principal\n\n"
                  "Returns an integer value -- a combination of flags "
                  "which are elements of attributes enum.\n"
            )
        .def("set_princ_attributes"
                , &py_kadm5::kadmin::set_princ_attributes
                , "Set attributes of principal\n\n"
                  "Neets two parameter: principal name (a string) and"
                  "new attributes -- a combination of flags "
                  "which are elements of attributes enum.\n"
            )
        .def("get_princ_keys"
                , &py_kadm5::kadmin::get_princ_keys
                , "Get a principal keys.\n\n"
                  "The keys are returned as a list of 3-tuples "
                  "(kvno, enctype, key), where kvno is positive "
                  "integer, enctype is a number, and key is "
                  "decripted principal key stored in python string "
                  "for efficiency.\n"
            )
        .def("create_princ"
                , &py_kadm5::kadmin::create_princ
                , "Create principal.\n\n"
                , (arg("args") = dict())
            )
        .def("create_service_princ"
                , &py_kadm5::kadmin::create_service_princ
                , "Create service principal with a random key.\n\n"
                  "New service principal is created. Service principals "
                  "in kerberos follow the naming convention:\n"
                  "\t<service>/<hostname>@<REALM>\n"
                  "This function takes service name and a host name "
                  "and constructs a principal name in this form. "
                  "If no hostname is given (None, which is the default), "
                  "fqdn of localhost is used.\n"
                  "For created principal a random key is generated.\n"
                , (
                      arg("host") = (const char*) 0
                    , arg("args") = dict()
                  )
            )
        .def("get_realm"
                , &py_kadm5::kadmin::get_realm
                , "Get realm name."
            )
        ;


    class_<py_kadm5::keytab, boost::noncopyable>(
              "keytab"
            , "Kerberos 5 keytab management."
            , init<object, const char*>(
                  "initialize keytab object\n"
                , (
                      arg("error_cb") = object()
                    , arg("name") = (const char*) 0
                  )
                )
            )
        .def("list"
                , &py_kadm5::keytab::list
                , "Retrieve a list of keytab entries.\n\n"
                  "The list consists of 3-tuples (kvno, enctype, princname)\n"
            )
        .def("add_entry"
                , &py_kadm5::keytab::add_entry
                , "Add one entry to keytab.\n\n"
                  "The first argument must be a full principal name.\n"
            )
        .def("remove_princ"
                , &py_kadm5::keytab::remove_princ
                , "Remove keys of given principal from the keytab.\n\n"
                  "If kvno = 0 (the default) all keys of that principal "
                  "are removed. If kvno != 0, only keys with version number "
                  "equal to kvno are removed.\n\n"
                  "The function returns number of successfully delted keys.\n"
                , (
                      arg("kvno") =  0
                    , arg("enctype") = 0
                  )
            )
        ;
}


